@@ -6,7 +6,7 @@ for stage in (1, 2, 3, 5, 7)
 
     @eval begin
         """
-              $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
+            $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
                       defect_threshold = 0.1, max_num_subintervals = 3000)
 
         $($stage)th stage RadauIIa method.
@@ -17,6 +17,9 @@ for stage in (1, 2, 3, 5, 7)
             `NonlinearProblem` interface can be used. Note that any autodiff argument for
             the solver will be ignored and a custom jacobian algorithm will be used.
 
+          - `optimize`: Internal Optimization solver. Any solver which conforms to the SciML
+            `OptimizationProblem` interface can be used. Note that any autodiff argument for
+            the solver will be ignored and a custom jacobian algorithm will be used.
           - `jac_alg`: Jacobian Algorithm used for the nonlinear solver. Defaults to
             `BVPJacobianAlgorithm()`, which automatically decides the best algorithm to
             use based on the input types and problem type.
@@ -87,23 +90,15 @@ for stage in (1, 2, 3, 5, 7)
         }
         ```
         """
-        Base.@kwdef struct $(alg){N, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
+        Base.@kwdef struct $(alg){N, O, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
             nlsolve::N = nothing
+            optimize::O = nothing
             jac_alg::J = BVPJacobianAlgorithm()
             nested_nlsolve::Bool = false
             nested_nlsolve_kwargs::NamedTuple = (;)
             defect_threshold::T = 0.1
             max_num_subintervals::Int = 3000
         end
-        $(alg)(nlsolve::N,
-            jac_alg::J;
-            nested = false,
-            nested_nlsolve_kwargs::NamedTuple = (;),
-            defect_threshold::T = 0.1,
-            max_num_subintervals::Int = 3000) where {N,
-            J,
-            T} = $(alg){N, J, T}(nlsolve, jac_alg, nested, nested_nlsolve_kwargs,
-            defect_threshold, max_num_subintervals)
     end
 end
 
@@ -112,17 +107,20 @@ for stage in (2, 3, 4, 5)
 
     @eval begin
         """
-              $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
+            $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
                       defect_threshold = 0.1, max_num_subintervals = 3000)
 
         $($stage)th stage LobattoIIIa method.
 
         ## Keyword Arguments
 
-            - `nlsolve`: Internal Nonlinear solver. Any solver which conforms to the SciML
+          - `nlsolve`: Internal Nonlinear solver. Any solver which conforms to the SciML
             `NonlinearProblem` interface can be used. Note that any autodiff argument for
-              the solver will be ignored and a custom jacobian algorithm will be used.
+            the solver will be ignored and a custom jacobian algorithm will be used.
 
+          - `optimize`: Internal Optimization solver. Any solver which conforms to the SciML
+            `OptimizationProblem` interface can be used. Note that any autodiff argument for
+            the solver will be ignored and a custom jacobian algorithm will be used.
           - `jac_alg`: Jacobian Algorithm used for the nonlinear solver. Defaults to
             `BVPJacobianAlgorithm()`, which automatically decides the best algorithm to
             use based on the input types and problem type.
@@ -133,7 +131,6 @@ for stage in (2, 3, 4, 5)
                 `nonbc_diffmode` defaults to `AutoSparse(AutoForwardDiff)` if possible else
                 `AutoSparse(AutoFiniteDiff)`. For `bc_diffmode`, defaults to `AutoForwardDiff` if
                 possible else `AutoFiniteDiff`.
-
           - `nested_nlsolve`: Whether or not to use a nested nonlinear solve for the
             implicit FIRK step. Defaults to `false`. If set to `false`, the FIRK stages are
             solved as a part of the global residual. The general recommendation is to choose
@@ -150,65 +147,59 @@ for stage in (2, 3, 4, 5)
 
         ## References
 
-              Reference for Lobatto and Radau methods:
-              ```bibtex
-                  @Inbook{Jay2015,
-                  author="Jay, Laurent O.",
-                  editor="Engquist, Bj{\"o}rn",
-                  title="Lobatto Methods",
-                  booktitle = {Encyclopedia of {Applied} and {Computational} {Mathematics}},
-                  year="2015",
-                  publisher="Springer Berlin Heidelberg",
-                  }
-                  @incollection{engquist_radau_2015,
-                  author = {Hairer, Ernst and Wanner, Gerhard},
-                  title = {Radau {Methods}},
-                  booktitle = {Encyclopedia of {Applied} and {Computational} {Mathematics}},
-                  publisher = {Springer Berlin Heidelberg},
-                  editor="Engquist, Bj{\"o}rn",
-                  year = {2015},
-              }
-              ```
-              References for implementation of defect control, based on the `bvp5c` solver in MATLAB:
-              ```bibtex
-              @article{shampine_solving_nodate,
-              title = {Solving {Boundary} {Value} {Problems} for {Ordinary} {Diﬀerential} {Equations} in {Matlab} with bvp4c
-              author = {Shampine, Lawrence F and Kierzenka, Jacek and Reichelt, Mark W},
-              year = {2000},
-              }
+        Reference for Lobatto and Radau methods:
 
-              @article{kierzenka_bvp_2008,
-                  title = {A {BVP} {Solver} that {Controls} {Residual} and {Error}},
-                  author = {Kierzenka, J and Shampine, L F},
-                  year = {2008},
-              }
+        ```bibtex
+        @Inbook{Jay2015,
+            author="Jay, Laurent O.",
+            editor="Engquist, Bj{\"o}rn",
+            title="Lobatto Methods",
+            booktitle = {Encyclopedia of {Applied} and {Computational} {Mathematics}},
+            year="2015",
+            publisher="Springer Berlin Heidelberg",
+        }
+        @incollection{engquist_radau_2015,
+            author = {Hairer, Ernst and Wanner, Gerhard},
+            title = {Radau {Methods}},
+            booktitle = {Encyclopedia of {Applied} and {Computational} {Mathematics}},
+            publisher = {Springer Berlin Heidelberg},
+            editor="Engquist, Bj{\"o}rn",
+            year = {2015},
+        }
+        ```
 
-              @article{russell_adaptive_1978,
-                  title = {Adaptive {Mesh} {Selection} {Strategies} for {Solving} {Boundary} {Value} {Problems}},
-                  journal = {SIAM Journal on Numerical Analysis},
-                  author = {Russell, R. D. and Christiansen, J.},
-                  year = {1978},
-                  file = {Russell and Christiansen - 1978 - Adaptive Mesh Selection Strategies for Solving Bou.pdf:/Users/AXLRSN/Zotero/storage/HKU27A4T/Russell and Christiansen - 1978 - Adaptive Mesh Selection Strategies for Solving Bou.pdf:application/pdf},
-              }
-              ```
+        References for implementation of defect control, based on the `bvp5c` solver in MATLAB:
+
+        ```bibtex
+        @article{shampine_solving_nodate,
+            title = {Solving {Boundary} {Value} {Problems} for {Ordinary} {Diﬀerential} {Equations} in {Matlab} with bvp4c
+            author = {Shampine, Lawrence F and Kierzenka, Jacek and Reichelt, Mark W},
+            year = {2000},
+        }
+
+        @article{kierzenka_bvp_2008,
+            title = {A {BVP} {Solver} that {Controls} {Residual} and {Error}},
+            author = {Kierzenka, J and Shampine, L F},
+            year = {2008},
+        }
+
+        @article{russell_adaptive_1978,
+            title = {Adaptive {Mesh} {Selection} {Strategies} for {Solving} {Boundary} {Value} {Problems}},
+            journal = {SIAM Journal on Numerical Analysis},
+            author = {Russell, R. D. and Christiansen, J.},
+            year = {1978},
+        }
+        ```
         """
-        Base.@kwdef struct $(alg){N, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
+        Base.@kwdef struct $(alg){N, O, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
             nlsolve::N = nothing
+            optimize::O = nothing
             jac_alg::J = BVPJacobianAlgorithm()
             nested_nlsolve::Bool = false
             nested_nlsolve_kwargs::NamedTuple = (;)
             defect_threshold::T = 0.1
             max_num_subintervals::Int = 3000
         end
-        $(alg)(nlsolve::N,
-            jac_alg::J;
-            nested = false,
-            nested_nlsolve_kwargs::NamedTuple = (;),
-            defect_threshold::T = 0.1,
-            max_num_subintervals::Int = 3000) where {N,
-            J,
-            T} = $(alg){N, J, T}(nlsolve, jac_alg, nested, nested_nlsolve_kwargs,
-            defect_threshold, max_num_subintervals)
     end
 end
 
@@ -217,7 +208,7 @@ for stage in (2, 3, 4, 5)
 
     @eval begin
         """
-              $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
+            $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
                       defect_threshold = 0.1, max_num_subintervals = 3000)
 
         $($stage)th stage LobattoIIIb method.
@@ -228,6 +219,9 @@ for stage in (2, 3, 4, 5)
             `NonlinearProblem` interface can be used. Note that any autodiff argument for
             the solver will be ignored and a custom jacobian algorithm will be used.
 
+          - `optimize`: Internal Optimization solver. Any solver which conforms to the SciML
+            `OptimizationProblem` interface can be used. Note that any autodiff argument for
+            the solver will be ignored and a custom jacobian algorithm will be used.
           - `jac_alg`: Jacobian Algorithm used for the nonlinear solver. Defaults to
             `BVPJacobianAlgorithm()`, which automatically decides the best algorithm to
             use based on the input types and problem type.
@@ -299,23 +293,15 @@ for stage in (2, 3, 4, 5)
         }
         ```
         """
-        Base.@kwdef struct $(alg){N, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
+        Base.@kwdef struct $(alg){N, O, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
             nlsolve::N = nothing
+            optimize::O = nothing
             jac_alg::J = BVPJacobianAlgorithm()
             nested_nlsolve::Bool = false
             nested_nlsolve_kwargs::NamedTuple = (;)
             defect_threshold::T = 0.1
             max_num_subintervals::Int = 3000
         end
-        $(alg)(nlsolve::N,
-            jac_alg::J;
-            nested = false,
-            nested_nlsolve_kwargs::NamedTuple = (;),
-            defect_threshold::T = 0.1,
-            max_num_subintervals::Int = 3000) where {N,
-            J,
-            T} = $(alg){N, J, T}(nlsolve, jac_alg, nested, nested_nlsolve_kwargs,
-            defect_threshold, max_num_subintervals)
     end
 end
 
@@ -324,7 +310,7 @@ for stage in (2, 3, 4, 5)
 
     @eval begin
         """
-              $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
+            $($alg)(; nlsolve = NewtonRaphson(), jac_alg = BVPJacobianAlgorithm(), nested_nlsolve = false, nest_tol = 0.0,
                       defect_threshold = 0.1, max_num_subintervals = 3000)
 
         $($stage)th stage LobattoIIIc method.
@@ -335,6 +321,9 @@ for stage in (2, 3, 4, 5)
             `NonlinearProblem` interface can be used. Note that any autodiff argument for
             the solver will be ignored and a custom jacobian algorithm will be used.
 
+          - `optimize`: Internal Optimization solver. Any solver which conforms to the SciML
+            `OptimizationProblem` interface can be used. Note that any autodiff argument for
+            the solver will be ignored and a custom jacobian algorithm will be used.
           - `jac_alg`: Jacobian Algorithm used for the nonlinear solver. Defaults to
             `BVPJacobianAlgorithm()`, which automatically decides the best algorithm to
             use based on the input types and problem type.
@@ -406,23 +395,15 @@ for stage in (2, 3, 4, 5)
         }
         ```
         """
-        Base.@kwdef struct $(alg){N, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
+        Base.@kwdef struct $(alg){N, O, J <: BVPJacobianAlgorithm, T} <: AbstractFIRK
             nlsolve::N = nothing
+            optimize::O = nothing
             jac_alg::J = BVPJacobianAlgorithm()
             nested_nlsolve::Bool = false
             nested_nlsolve_kwargs::NamedTuple = (;)
             defect_threshold::T = 0.1
             max_num_subintervals::Int = 3000
         end
-        $(alg)(nlsolve::N,
-            jac_alg::J;
-            nested = false,
-            nested_nlsolve_kwargs::NamedTuple = (;),
-            defect_threshold::T = 0.1,
-            max_num_subintervals::Int = 3000) where {N,
-            J,
-            T} = $(alg){N, J, T}(nlsolve, jac_alg, nested, nested_nlsolve_kwargs,
-            defect_threshold, max_num_subintervals)
     end
 end
 
