@@ -5,17 +5,18 @@ struct FIRKExpandInterpolation{T1, T2} <: AbstractDiffEqInterpolation
     cache
 end
 
-function DiffEqBase.interp_summary(interp::FIRKExpandInterpolation)
+function SciMLBase.interp_summary(interp::FIRKExpandInterpolation)
     return "FIRK Order $(interp.cache.order) Interpolation"
 end
 
 function (id::FIRKExpandInterpolation)(tvals, idxs, deriv, p, continuity::Symbol = :left)
-    interpolation(tvals, id, idxs, deriv, p, continuity)
+    return interpolation(tvals, id, idxs, deriv, p, continuity)
 end
 
 function (id::FIRKExpandInterpolation)(
-        val, tvals, idxs, deriv, p, continuity::Symbol = :left)
-    interpolation!(val, tvals, id, idxs, deriv, p, continuity)
+        val, tvals, idxs, deriv, p, continuity::Symbol = :left
+    )
+    return interpolation!(val, tvals, id, idxs, deriv, p, continuity)
 end
 
 # FIRK Nested Interpolation
@@ -25,21 +26,24 @@ struct FIRKNestedInterpolation{T1, T2} <: AbstractDiffEqInterpolation
     cache
 end
 
-function DiffEqBase.interp_summary(interp::FIRKNestedInterpolation)
+function SciMLBase.interp_summary(interp::FIRKNestedInterpolation)
     return "FIRK Order $(interp.cache.order) Interpolation"
 end
 
 function (id::FIRKNestedInterpolation)(tvals, idxs, deriv, p, continuity::Symbol = :left)
-    interpolation(tvals, id, idxs, deriv, p, continuity)
+    return interpolation(tvals, id, idxs, deriv, p, continuity)
 end
 
 function (id::FIRKNestedInterpolation)(
-        val, tvals, idxs, deriv, p, continuity::Symbol = :left)
-    interpolation!(val, tvals, id, idxs, deriv, p, continuity)
+        val, tvals, idxs, deriv, p, continuity::Symbol = :left
+    )
+    return interpolation!(val, tvals, id, idxs, deriv, p, continuity)
 end
 
-@inline function interpolation(tvals, id::FIRKNestedInterpolation, idxs,
-        deriv::D, p, continuity::Symbol = :left) where {D}
+@inline function interpolation(
+        tvals, id::FIRKNestedInterpolation, idxs,
+        deriv::D, p, continuity::Symbol = :left
+    ) where {D}
     (; t, u, cache) = id
     tdir = sign(t[end] - t[1])
     idx = sortperm(tvals, rev = tdir < 0)
@@ -60,8 +64,10 @@ end
     return DiffEqArray(vals, tvals)
 end
 
-@inline function interpolation!(vals, tvals, id::FIRKNestedInterpolation, idxs,
-        deriv::D, p, continuity::Symbol = :left) where {D}
+@inline function interpolation!(
+        vals, tvals, id::FIRKNestedInterpolation, idxs,
+        deriv::D, p, continuity::Symbol = :left
+    ) where {D}
     (; t, cache) = id
     tdir = sign(t[end] - t[1])
     idx = sortperm(tvals, rev = tdir < 0)
@@ -71,10 +77,13 @@ end
         interpolant!(z, id.cache, tvals[j], id.cache.mesh, id.cache.mesh_dt, deriv)
         vals[j] = z
     end
+    return
 end
 
-@inline function interpolation(tval::Number, id::FIRKNestedInterpolation, idxs,
-        deriv::D, p, continuity::Symbol = :left) where {D}
+@inline function interpolation(
+        tval::Number, id::FIRKNestedInterpolation, idxs,
+        deriv::D, p, continuity::Symbol = :left
+    ) where {D}
     z = similar(id.cache.fᵢ₂_cache)
     interpolant!(z, id.cache, tval, id.cache.mesh, id.cache.mesh_dt, deriv)
     return idxs !== nothing ? z[idxs] : z
@@ -82,7 +91,8 @@ end
 
 @inline function interpolant!(
         z::AbstractArray, cache::FIRKCacheNested{iip, T, diffcache, fit_parameters},
-        t, mesh, mesh_dt, ::Type{Val{0}}) where {iip, T, diffcache, fit_parameters}
+        t, mesh, mesh_dt, ::Type{Val{0}}
+    ) where {iip, T, diffcache, fit_parameters}
     (; f, ITU, nest_prob, alg) = cache
     (; q_coeff) = ITU
 
@@ -95,7 +105,7 @@ end
     τ = (t - mesh[j])
     length_z = length(z)
 
-    nest_nlsolve_alg = __concrete_nonlinearsolve_algorithm(nest_prob, alg.nlsolve)
+    nest_nlsolve_alg = __concrete_solve_algorithm(nest_prob, alg.nlsolve)
     nestprob_p = zeros(T, cache.M + 2)
 
     yᵢ = copy(cache.y[j].du)
@@ -114,7 +124,7 @@ end
 
     nestprob_p[1] = mesh[j]
     nestprob_p[2] = mesh_dt[j]
-    nestprob_p[3:end] .= ifelse(fit_parameters, vcat(yᵢ, cache.p), yᵢ)
+    nestprob_p[3:end] .= ifelse(fit_parameters, vcat(yᵢ, __tunable_part(cache.p)), yᵢ)
 
     _nestprob = remake(nest_prob, p = nestprob_p)
     nestsol = __solve(_nestprob, nest_nlsolve_alg; alg.nested_nlsolve_kwargs...)
@@ -123,12 +133,13 @@ end
     z₁, z₁′ = eval_q(yᵢ, 0.5, h, q_coeff, @view(K[1:length_z, :])) # Evaluate q(x) at midpoints
     S_coeffs = get_S_coeffs(h, yᵢ, yᵢ₊₁, z₁, dyᵢ, dyᵢ₊₁, z₁′)
 
-    S_interpolate!(z, τ, S_coeffs)
+    return S_interpolate!(z, τ, S_coeffs)
 end
 
 @inline function interpolant!(
         dz::AbstractArray, cache::FIRKCacheNested{iip, T, diffcache, fit_parameters},
-        t, mesh, mesh_dt, ::Type{Val{1}}) where {iip, T, diffcache, fit_parameters}
+        t, mesh, mesh_dt, ::Type{Val{1}}
+    ) where {iip, T, diffcache, fit_parameters}
     (; f, ITU, nest_prob, alg) = cache
     (; q_coeff) = ITU
 
@@ -141,7 +152,7 @@ end
     τ = (t - mesh[j])
     length_dz = length(dz)
 
-    nest_nlsolve_alg = __concrete_nonlinearsolve_algorithm(nest_prob, alg.nlsolve)
+    nest_nlsolve_alg = __concrete_solve_algorithm(nest_prob, alg.nlsolve)
     nestprob_p = zeros(T, cache.M + 2)
 
     yᵢ = copy(cache.y[j].du)
@@ -160,7 +171,7 @@ end
 
     nestprob_p[1] = mesh[j]
     nestprob_p[2] = mesh_dt[j]
-    nestprob_p[3:end] .= ifelse(fit_parameters, vcat(yᵢ, cache.p), yᵢ)
+    nestprob_p[3:end] .= ifelse(fit_parameters, vcat(yᵢ, __tunable_part(cache.p)), yᵢ)
 
     _nestprob = remake(nest_prob, p = nestprob_p)
     nestsol = __solve(_nestprob, nest_nlsolve_alg; alg.nested_nlsolve_kwargs...)
@@ -169,12 +180,14 @@ end
     z₁, z₁′ = eval_q(yᵢ, 0.5, h, q_coeff, @view(K[1:length_dz, :]))
     S_coeffs = get_S_coeffs(h, yᵢ, yᵢ₊₁, z₁, dyᵢ, dyᵢ₊₁, z₁′)
 
-    dS_interpolate!(dz, τ, S_coeffs)
+    return dS_interpolate!(dz, τ, S_coeffs)
 end
 
 ## Expanded
-@inline function interpolation(tvals, id::FIRKExpandInterpolation, idxs,
-        deriv::D, p, continuity::Symbol = :left) where {D}
+@inline function interpolation(
+        tvals, id::FIRKExpandInterpolation, idxs,
+        deriv::D, p, continuity::Symbol = :left
+    ) where {D}
     (; t, u, cache) = id
     tdir = sign(t[end] - t[1])
     idx = sortperm(tvals, rev = tdir < 0)
@@ -195,8 +208,10 @@ end
     return DiffEqArray(vals, tvals)
 end
 
-@inline function interpolation!(vals, tvals, id::FIRKExpandInterpolation, idxs,
-        deriv::D, p, continuity::Symbol = :left) where {D}
+@inline function interpolation!(
+        vals, tvals, id::FIRKExpandInterpolation, idxs,
+        deriv::D, p, continuity::Symbol = :left
+    ) where {D}
     (; t, cache) = id
     tdir = sign(t[end] - t[1])
     idx = sortperm(tvals, rev = tdir < 0)
@@ -206,17 +221,22 @@ end
         interpolant!(z, id.cache, tvals[j], id.cache.mesh, id.cache.mesh_dt, deriv)
         vals[j] = z
     end
+    return
 end
 
-@inline function interpolation(tval::Number, id::FIRKExpandInterpolation, idxs,
-        deriv::D, p, continuity::Symbol = :left) where {D}
+@inline function interpolation(
+        tval::Number, id::FIRKExpandInterpolation, idxs,
+        deriv::D, p, continuity::Symbol = :left
+    ) where {D}
     z = similar(id.cache.fᵢ₂_cache)
     interpolant!(z, id.cache, tval, id.cache.mesh, id.cache.mesh_dt, deriv)
     return idxs !== nothing ? z[idxs] : z
 end
 
-@inline function interpolant!(z::AbstractArray, cache::FIRKCacheExpand{iip},
-        t, mesh, mesh_dt, ::Type{Val{0}}) where {iip}
+@inline function interpolant!(
+        z::AbstractArray, cache::FIRKCacheExpand{iip},
+        t, mesh, mesh_dt, ::Type{Val{0}}
+    ) where {iip}
     j = interval(mesh, t)
     h = mesh_dt[j]
     lf = (length(cache.y₀) - 1) / (length(cache.y) - 1)
@@ -255,11 +275,13 @@ end
     z₁, z₁′ = eval_q(yᵢ, 0.5, h, q_coeff, @view(K[1:length_z, :])) # Evaluate q(x) at midpoints
     S_coeffs = get_S_coeffs(h, yᵢ, yᵢ₊₁, z₁, dyᵢ, dyᵢ₊₁, z₁′)
 
-    S_interpolate!(z, τ, S_coeffs)
+    return S_interpolate!(z, τ, S_coeffs)
 end
 
-@inline function interpolant!(dz::AbstractArray, cache::FIRKCacheExpand{iip},
-        t, mesh, mesh_dt, ::Type{Val{1}}) where {iip}
+@inline function interpolant!(
+        dz::AbstractArray, cache::FIRKCacheExpand{iip},
+        t, mesh, mesh_dt, ::Type{Val{1}}
+    ) where {iip}
     j = interval(mesh, t)
     h = mesh_dt[j]
     lf = (length(cache.y₀) - 1) / (length(cache.y) - 1)
@@ -298,15 +320,19 @@ end
     z₁, z₁′ = eval_q(yᵢ, 0.5, h, q_coeff, @view(K[1:length_dz, :])) # Evaluate q(x) at midpoints
     S_coeffs = get_S_coeffs(h, yᵢ, yᵢ₊₁, z₁, dyᵢ, dyᵢ₊₁, z₁′)
 
-    dS_interpolate!(dz, τ, S_coeffs)
+    return dS_interpolate!(dz, τ, S_coeffs)
 end
 
-@inline __build_interpolation(cache::FIRKCacheExpand,
-    u::AbstractVector) = FIRKExpandInterpolation(cache.mesh, u, cache)
-@inline __build_interpolation(cache::FIRKCacheNested,
-    u::AbstractVector) = FIRKNestedInterpolation(cache.mesh, u, cache)
+@inline __build_interpolation(
+    cache::FIRKCacheExpand,
+    u::AbstractVector
+) = FIRKExpandInterpolation(cache.mesh, u, cache)
+@inline __build_interpolation(
+    cache::FIRKCacheNested,
+    u::AbstractVector
+) = FIRKNestedInterpolation(cache.mesh, u, cache)
 
-# Intermidiate solution for evaluating boundry conditions
+# Intermediate solution for evaluating boundary conditions
 # basically simplified version of the interpolation for FIRK
 # Expanded FIRK
 function (s::EvalSol{C})(tval::Number) where {C <: FIRKCacheExpand}
@@ -351,7 +377,7 @@ function (s::EvalSol{C})(tval::Number) where {C <: FIRKCacheExpand}
         z₁′[i] = sum(ii * coeffs[ii] * (τ * h)^(ii - 1) for ii in axes(coeffs, 1))
     end
 
-    S_coeffs = get_S_coeffs_interp(h, yᵢ, yᵢ₊₁, z₁, dyᵢ, dyᵢ₊₁, z₁′)
+    S_coeffs = get_S_coeffs(h, yᵢ, yᵢ₊₁, z₁, dyᵢ, dyᵢ₊₁, z₁′)
 
     z = similar(yᵢ)
 
@@ -359,14 +385,6 @@ function (s::EvalSol{C})(tval::Number) where {C <: FIRKCacheExpand}
     return z
 end
 
-function get_S_coeffs_interp(h, yᵢ, yᵢ₊₁, dyᵢ, dyᵢ₊₁, ymid, dymid)
-    vals = vcat(yᵢ, yᵢ₊₁, dyᵢ, dyᵢ₊₁, ymid, dymid)
-    M = length(yᵢ)
-    A = s_constraints_interp(M, h)
-    coeffs = reshape(A \ vals, 6, M)'
-    return coeffs
-end
-
 function get_q_coeffs_interp(A, ki, h)
     coeffs = A * ki
     for i in axes(coeffs, 1)
@@ -375,30 +393,6 @@ function get_q_coeffs_interp(A, ki, h)
     return coeffs
 end
 
-function s_constraints_interp(M, h)
-    t = repeat([0.0, 1.0 * h, 0.5 * h, 0.0, 1.0 * h, 0.5 * h], M)
-    A = zeros(6 * M, 6 * M)
-
-    for i in 1:6
-        row_start = (i - 1) * M + 1
-        for k in 0:(M - 1)
-            for j in 1:6
-                A[row_start + k, j + k * 6] = t[i + k * 6]^(j - 1)
-            end
-        end
-    end
-    for i in 4:6
-        row_start = (i - 1) * M + 1
-        for k in 0:(M - 1)
-            for j in 1:6
-                A[row_start + k,
-                    j + k * 6] = j == 1.0 ? 0.0 : (j - 1) * t[i + k * 6]^(j - 2)
-            end
-        end
-    end
-
-    return A
-end
 
 # Nested FIRK
 function (s::EvalSol{C})(tval::Number) where {C <: FIRKCacheNested}
@@ -413,7 +407,7 @@ function (s::EvalSol{C})(tval::Number) where {C <: FIRKCacheNested}
     h = mesh_dt[j]
     τ = tval - t[j]
 
-    nest_nlsolve_alg = __concrete_nonlinearsolve_algorithm(nest_prob, alg.nlsolve)
+    nest_nlsolve_alg = __concrete_solve_algorithm(nest_prob, alg.nlsolve)
     nestprob_p = zeros(cache.M + 2)
 
     yᵢ = u[j]
